@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/c99eval.scm - evaluate constant expressions
 
-;; Copyright (C) 2018-2020 Matthew R. Wette
+;; Copyright (C) 2018-2021 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -20,10 +20,15 @@
 (define-module (nyacc lang c99 cxeval)
   #:export (parse-c99-cx
 	    eval-c99-cx
+	    size-and-align-of-type
 	    eval-sizeof-type
+	    eval-alignof-type
 	    eval-sizeof-expr
+	    eval-offsetof
 	    sizeof-mtail
-	    )
+	    cx-incr-size
+	    cx-maxi-size
+	    find-offsets)
   #:use-module (nyacc lalr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
@@ -33,9 +38,9 @@
   #:use-module (nyacc lang arch-info)
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lang c99 parser)
+  #:use-module (nyacc lang c99 munge)
   #:use-module (nyacc lang c99 munge-base)
   #:use-module (rnrs arithmetic bitwise)
-  ;;#:use-module ((srfi srfi-43) #:select (vector-map vector-for-each))
   #:use-module (system foreign)
   #:use-module (ice-9 match))
 
@@ -45,28 +50,6 @@
 (define (pperr exp)
   (pretty-print exp (current-error-port) #:per-line-prefix "  "))
 
-(define ffi-type-map
-  `(("void" . ,void) ("float" . ,float) ("double" . ,double) ("short" . ,short)
-    ("short int" . ,short) ("signed short" . ,short)
-    ("signed short int" . ,short) ("int" . ,int) ("signed" . ,int)
-    ("signed int" . ,int) ("long" . ,long) ("long int" . ,long)
-    ("signed long" . ,long) ("signed long int" . ,long)
-    ("unsigned short int" . ,unsigned-short)
-    ("unsigned short" . ,unsigned-short)
-    ("unsigned int" . ,unsigned-int) ("unsigned" . ,unsigned-int)
-    ("unsigned long int" . ,unsigned-long) ("unsigned long" . ,unsigned-long)
-    ("char" . ,int8) ("signed char" . ,int8) ("unsigned char" . ,uint8)
-    ("wchar_t" . ,int) ("char16_t" . ,int16) ("char32_t" . ,int32)
-    ("long long" . ,long) ("long long int" . ,long)
-    ("signed long long" . ,long) ("signed long long int" . ,long)
-    ("unsigned long long" . ,unsigned-long)
-    ("unsigned long long int" . ,unsigned-long) ("_Bool" . ,int8)
-    ("size_t" . ,size_t)))
-
-(define (sizeof-type name)
-  (or (and=> (assoc-ref ffi-type-map name) sizeof)
-      (throw 'c99-error "bad type")))
-
 ;; (string "abc" "dev")
 (define (sizeof-string-const value)
   #f)
@@ -103,89 +86,77 @@
 	 (xname (and xdecl (sx-ref* xdecl 1 1 1 1))))
     xname))
 
-(define (sizeof-mtail mtail arch)
-  ;; (decl attr specl (declr-list declr1 declr2 ...))
-  ;; => ((decl attr specl declr1) (decl attr specl declr2) ...)
-  (define (splitup-decl decl)
-    (let ((tag (sx-tag decl)) (attr (sx-attr decl))
-	  (specl (sx-ref decl 1)) (dclrl (sx-ref decl 2)))
-      (map (lambda (declr) (sx-list tag attr specl declr)) (sx-tail dclrl))))
+;; Update struct running-size (rs) given new item size (s) and align't (a).
+(define (cx-incr-size s a rs)
+  (+ s (* a (quotient (+ rs (1- a)) a))))
 
-  ;; Update running struct size (rs) given new item s and a.
-  (define (incr-size s a rs)
-    (+ s (* a (quotient (+ rs (1- a)) a))))
+;; Update running union size (rs) given new item s and a.
+(define (cx-maxi-size s a rs)
+  (max s rs))
 
-  ;; Update running union size (rs) given new item s and a.
-  (define (maxi-size s a rs)
-    (max s rs))
+(define incr-size cx-incr-size)
+(define maxi-size cx-maxi-size)
+
+(define (sizeof-mtail mtail)
 
   (define (exec/decl decl size base-align update)
     (let ((mtail (sx-tail (sx-find 'type-spec (sx-ref decl 1))))
-	  (declrs (sx-tail (sx-ref decl 2))))
+	  (declrs (or (and=> (sx-ref decl 2) sx-tail) '((ident "_")))))
       (let loop ((size size) (base-align base-align) (declrs declrs))
 	(if (null? declrs)
 	    (values size base-align)
 	    (call-with-values
 		(lambda ()
 		  (sizeof-mtail
-		   (cdr (m-unwrap-declr (car declrs) mtail)) arch))
+		   (cdr (m-unwrap-declr (car declrs) mtail))))
 	      (lambda (elt-size elt-align)
 		(loop (update elt-size elt-align size)
 		      (max elt-align base-align) (cdr declrs))))))))
 
-  (define (incr/decl decl size base-align)
-    (exec/decl decl size base-align incr-size))
-  
-  (define (maxi/decl decl size base-align)
-    (exec/decl decl size base-align maxi-size))
-
   (match mtail
     (`((pointer-to) . ,rest)
-     (values (sizeof-basetype '* arch) (alignof-basetype '* arch)))
+     (values (sizeof-basetype '*) (alignof-basetype '*)))
     (`((fixed-type ,name))
-     (values (sizeof-basetype name arch) (alignof-basetype name arch)))
+     (values (sizeof-basetype name) (alignof-basetype name)))
     (`((float-type ,name))
-     (values (sizeof-basetype name arch) (alignof-basetype name arch)))
+     (values (sizeof-basetype name) (alignof-basetype name)))
     (`((array-of ,size) . ,rest)
      (let ((mult (eval-c99-cx size)))
        (call-with-values
-	   (lambda () (sizeof-mtail rest arch))
+	   (lambda () (sizeof-mtail rest))
 	 (lambda (size align)
 	   (values (* mult size) align)))))
-
     (`((struct-def (field-list . ,fields)))
      (let loop ((size 0) (align 0) (flds fields))
        (cond
 	((null? flds) (values (incr-size 0 align size) align))
 	((eq? 'comp-decl (sx-tag (car flds)))
 	 (call-with-values
-	     (lambda () (incr/decl (car flds) size align))
+	     ;;(lambda () (incr/decl (car flds) size align))
+	     (lambda () (exec/decl (car flds) size align incr-size))
 	   (lambda (size align)
 	     (loop size align (cdr flds)))))
 	(else (loop size align (cdr flds))))))
-
     (`((union-def (field-list . ,fields)))
      (let loop ((size 0) (align 0) (flds fields))
        (cond
 	((null? flds) (values (incr-size 0 align size) align))
 	((eq? 'comp-decl (sx-tag (car flds)))
 	 (call-with-values
-	     (lambda () (maxi/decl (car flds) size align))
+	     (lambda () (exec/decl (car flds) size align maxi-size))
 	   (lambda (size align)
 	     (loop size align (cdr flds)))))
 	(else (loop size align (cdr flds))))))
-
     (`((,(or 'enum-ref 'enum-def) . ,rest))
-     (values (sizeof-basetype "int" arch) (alignof-basetype "int" arch)))
-
+     (values (sizeof-basetype "int") (alignof-basetype "int")))
     (_ (sferr "c99/eval-sizeof-mtail: missed\n") (pperr mtail)
        (throw 'c99-error "coding error"))))
 
-(define* (sizeof-specl/declr specl declr #:optional (udict '()) #:key arch)
+(define* (sizeof-specl/declr specl declr #:optional (udict '()))
   (let* ((udecl `(udecl ,specl ,declr))
 	 (xdecl (expand-typerefs udecl udict))
 	 (mdecl (udecl->mdecl xdecl)))
-    (sizeof-mtail (cdr mdecl) arch)))
+    (sizeof-mtail (cdr mdecl))))
 
 (define (trim-mtail mtail)
   (case (caar mtail)
@@ -194,19 +165,35 @@
     ((initzer) (trim-mtail (cdr mtail)))
     (else mtail)))
 
-;; @deffn {Procedure} eval-sizeof-type tree [udict]
-;; => (values sizeof-val align-of)
+;; @deffn {Procedure} size-and-align-of-type tree [udict]
+;; @deffx {Procedure} eval-sizeof-type tree [udict]
+;; @deffx {Procedure} eval-alignof-type tree [udict]
+;; @example
+;; (size-and-align-of-type '(sizeof-type (ident "foo_t"))) => (values 4 2)
+;; (eval-sizeof-type '(sizeof-type (ident "foo_t"))) => 4
+;; (eval-alignof-type '(alignof-type (ident "foo_t"))) => 2
+;; @end example
 ;; @end deffn
-(define* (eval-sizeof-type tree #:optional (udict '()) #:key arch)
+(define* (size-and-align-of-type tree #:optional (udict '()))
   (let* ((type-name (sx-ref tree 1))
 	 (specl (sx-ref type-name 1))
 	 (declr (or (sx-ref type-name 2) '(param-declr))))
-    (sizeof-specl/declr specl declr udict #:arch arch)))
+    (sizeof-specl/declr specl declr udict)))
 
-;; @deffn {Procedure} eval-sizeof-expr tree [udict]
+(define* (eval-sizeof-type tree #:optional (udict '()))
+  (call-with-values
+      (lambda () (size-and-align-of-type tree udict))
+    (lambda (size align) size)))
+
+(define* (eval-alignof-type tree #:optional (udict '()))
+  (call-with-values
+      (lambda () (size-and-align-of-type tree udict))
+    (lambda (size align) align)))
+
+;; @deffn {Procedure} size-or-align-of-expr tree [udict]
 ;; => (values sizeof-val align-of)
 ;; @end deffn
-(define* (eval-sizeof-expr tree #:optional (udict '()) #:key arch)
+(define* (size-or-align-of-expr tree #:optional (udict '()))
 
   (define (sizeof-literal tree)
     (sx-match tree
@@ -243,8 +230,207 @@
 
   (or
    (sizeof-literal tree)
-   (sizeof-mtail (gen-mtail tree) arch)))
-      
+   (sizeof-mtail (gen-mtail tree))))
+
+(define* (eval-sizeof-expr tree #:optional (udict '()))
+  (call-with-values
+      (lambda () (size-or-align-of-expr tree udict))
+    (lambda (size align) size)))
+
+
+;; =============================================================================
+
+(define* (gen-offsets mtail #:optional (base 0))
+
+  (define (mt-al mtail)
+    (call-with-values (lambda () (sizeof-mtail mtail))
+      (lambda (sz al) al)))
+
+  (define (do-aggr flds update)
+    (let* ((aln (mt-al mtail)) (base (incr-size 0 aln base)))
+      (let loop ((siz 0) (aln aln) (offs '()) (decls '()) (flds flds))
+	(cond
+	 ((pair? decls)
+	  (let* ((mdecl (udecl->mdecl (car decls)))
+		 (name (car mdecl)) (mtail (cdr mdecl)))
+	    (call-with-values (lambda () (gen-offsets mtail (+ base siz)))
+	      (lambda (el-sz el-al el-os)
+		;;(sferr "sizeof(~S)=~S\n" name el-sz)
+		(let ((oval (if (pair? el-os) el-os
+				(+ base (incr-size 0 el-al siz)))))
+		  (loop (update el-sz el-al siz) (max aln el-al)
+			(acons name oval offs) (cdr decls) flds))))))
+	 ((pair? flds)
+	  (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
+	      (loop siz aln offs
+		    (map cdr (dictize-comp-decl (car flds))) (cdr flds))
+	      (loop siz aln offs decls (cdr flds))))
+	 (else (values siz aln (reverse offs)))))))
+
+  (match mtail
+    (`((pointer-to) . ,rest)
+     (let ((sz (sizeof-basetype '*)) (al (alignof-basetype '*)))
+       (values sz al (incr-size 0 al base))))
+    (`((fixed-type ,name))
+     (let ((sz (sizeof-basetype name)) (al (alignof-basetype name)))
+       (values sz al (incr-size 0 al base))))
+    (`((float-type ,name))
+     (let ((sz (sizeof-basetype name)) (al (alignof-basetype name)))
+       (values sz al (incr-size 0 al base))))
+    (`((array-of ,dim) . ,rest)
+     (call-with-values (lambda () (gen-offsets rest base))
+       (lambda (el-sz el-al el-of)
+	 (let ((base (incr-size 0 el-al base)))
+	   (let ((dim (eval-c99-cx dim)))
+	     (values (* dim el-sz) el-al (cons (cons dim el-sz) el-of)))))))
+    (`((struct-def (field-list . ,flds)) . ,rest)
+     (do-aggr flds incr-size))
+    (`((union-def (field-list . ,flds)) . ,rest)
+     (do-aggr flds maxi-size))
+    (`((,(or 'enum-ref 'enum-def) . ,rest))
+     (values (sizeof-basetype "int") (alignof-basetype "int") base))
+    (_ (sferr "c99/gen-offsets: missed\n") (pperr mtail)
+       (throw 'c99-error "coding error"))))
+
+(define (find-offsets mtail)
+  (call-with-values
+      (lambda () (gen-offsets mtail))
+    (lambda (size align offsets)
+      offsets)))
+  
+;; @deffn {Procedure} offsetof-mtail mtail desig [base] => offset alignment
+;; @end deffn
+(define* (offsetof-mtail mtail desig #:optional (base 0))
+
+  (define (exec/decl decl size align update)
+    ;; => values size align offs
+    (let ((mtail (sx-tail (sx-find 'type-spec (sx-ref decl 1))))
+	  (declrs (or (and=> (sx-ref decl 2) sx-tail) '((ident "_")))))
+      (let loop ((size size) (align align) (declrs declrs))
+	(cond
+	 ((null? declrs) (values size align))
+	 ((null? desig) (error "failed"))
+	 ((equal? (declr-ident (car declrs)) (car desig))
+	  (values (incr-size 0 align size) #f))
+	 (else
+	  (call-with-values
+	      (lambda ()
+		(let ((mdecl (m-unwrap-declr (car declrs) mtail)))
+		  (offsetof-mtail (cdr mdecl) desig base)))
+	    (lambda (elt-size elt-align)
+	      (loop (update elt-size elt-align size)
+		    (max elt-align align)
+		    (cdr declrs)))))))))
+
+  ;;(sferr "offsetof-mtail desig=~S  base=~S\n" desig base)
+  (match mtail
+
+    (`((array-of ,size) . ,rest)
+     (let ((mult (eval-c99-cx size)))
+       (call-with-values
+	   (lambda () (offsetof-mtail rest base))
+	 (lambda (size align offs) offs))))
+    
+    (`((struct-def (field-list . ,fields)))
+     (unless (equal? 'ident (caar desig))
+       (throw 'c99-error "cxeval: desig ~S not for struct" (car desig)))
+     (let loop ((offs base) (aln 0) (dsg (cadar desig))
+		(decls '()) (flds fields))
+       (cond
+	((pair? decls)
+	 (let* ((mdecl (udecl->mdecl (car decls)))
+		(name (car mdecl))
+		(mtail (cdr mdecl)))
+	   (call-with-values
+	       (lambda () (sizeof-mtail mtail))
+	     (lambda (elt-sz elt-al)
+	       (cond
+		((string=? name dsg)
+		 (let* ((xoffs (incr-size 0 elt-al offs)))
+		   (if (null? (cdr desig)) xoffs
+		       (offsetof-mtail mtail (cdr desig) xoffs))))
+		(else
+		 (loop (incr-size elt-sz elt-al offs) (max elt-al aln)
+		       dsg (cdr decls) flds)))))))
+	((pair? flds)
+	 (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
+	     (loop offs aln dsg
+		   (map cdr (dictize-comp-decl (car flds))) (cdr flds))
+	     (loop offs aln dsg decls (cdr flds))))
+	(else (values offs aln)))))
+	   
+    (`((union-def (field-list . ,fields)))
+     (unless (equal? 'ident (caar desig))
+       (throw 'c99-error "cxeval: desig ~S not for union" (car desig)))
+     (let loop ((offs base) (aln 0) (dsg (cadar desig))
+		(decls '()) (flds fields))
+       (cond
+	((pair? decls)
+	 (let* ((mdecl (udecl->mdecl (car decls)))
+		(name (car mdecl))
+		(mtail (cdr mdecl)))
+	   (call-with-values
+	       (lambda () (sizeof-mtail mtail))
+	     (lambda (elt-sz elt-al)
+	       (cond
+		((string=? name dsg)
+		 (let* ((xoffs (maxi-size 0 elt-al offs)))
+		   (if (null? (cdr desig)) xoffs
+		       (offsetof-mtail mtail (cdr desig) xoffs))))
+		(else
+		 (loop (maxi-size 0 elt-al offs) (max elt-al aln)
+		       dsg (cdr decls) flds)))))))
+	((pair? flds)
+	 (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
+	     (loop offs aln dsg
+		   (map cdr (dictize-comp-decl (car flds))) (cdr flds))
+	     (loop offs aln dsg decls (cdr flds))))
+	(else (values offs aln)))))
+
+    (_ (sferr "c99/eval-sizeof-mtail: missed\n") (pperr mtail)
+       (throw 'c99-error "coding error"))))
+
+(define* (unwrap-designator expr udict #:optional (seed '()))
+  (sx-match expr
+    ((p-expr (ident ,name)) (cons `(ident ,name) seed))
+    ((d-sel (ident ,elt) ,expr)
+     (unwrap-designator expr udict (cons `(ident ,elt) seed)))
+    ((array-ref ,ix ,expr)
+     (let ((ixval (eval-c99-cx ix udict)))
+       (unwrap-designator expr udict (cons `(ary-ref ,ixval) seed))))
+    (,_ (sferr "missed ~S\n" expr))))
+(export unwrap-designator)
+
+;; @deffn {Procedure} eval-offsetof tree [udict]
+;; where tree has the form
+;; @example
+;; (offsetof (type-name ...) designator-expr)
+;; e.g, (offsetof (type-name 
+;; @end example
+;; @example
+;;   offsetof(foo_t, designator)
+;; where
+;;   designator: ident | designator "." ident | designator "[" expr "]"
+;; @end example
+;; @end deffn
+(define* (eval-offsetof tree #:optional (udict '()))
+  (sx-match tree
+    ((offsetof-type (type-name ,decl-spec-list) ,expr)
+     (let* ((udecl `(udecl ,decl-spec-list (param-declr (ident "_"))))
+	    (xdecl (expand-typerefs udecl udict))
+	    (mdecl (udecl->mdecl xdecl))
+	    (desig (unwrap-designator expr udict)))
+       (offsetof-mtail (cdr mdecl) desig)))
+    (,_ #f)))
+
+
+;; =============================================================================
+
+;; (define (eval-typeof-type type-name)
+
+;; (define (eval-typeof-expr expr)
+
+;; =============================================================================
 
 ;; @deffn {Procedure} eval-c99-cx tree [udict] [#:fail-proc fail-proc]
 ;; Evaluate the constant expression or return #f
@@ -292,7 +478,7 @@
 	    ((pos) (and tree (ev1 tree)))
 	    ((neg) (uop - (ev1 tree)))
 	    ((not) (and tree (if (equal? 0 (ev1 tree)) 1 0)))
-	    ((mul) (bop * (ev1 tree) (ev2 tree)))
+
 	    ((div) (bop / (ev1 tree) (ev2 tree)))
 	    ((mod) (bop modulo (ev1 tree) (ev2 tree)))
 	    ((add) (bop + (ev1 tree) (ev2 tree)))
@@ -322,15 +508,19 @@
 	    ;;
 	    ((sizeof-type)
 	     (catch 'c99-error
-	       (lambda ()
-		 (call-with-values (lambda () (eval-sizeof-type tree udict))
-		   (lambda (size align) size)))
+	       (lambda () (eval-sizeof-type tree udict))
 	       (lambda (key fmt . args) (apply fail fmt args))))
 	    ((sizeof-expr)
 	     (catch 'c99-error
-	       (lambda ()
-		 (call-with-values (lambda () (eval-sizeof-expr tree udict))
-		   (lambda (size align) size)))
+	       (lambda () (eval-sizeof-expr tree udict))
+	       (lambda (key fmt . args) (apply fail fmt args))))
+	    ((alignof)
+	     (catch 'c99-error
+	       (lambda () (eval-alignof-type tree udict))
+	       (lambda (key fmt . args) (apply fail fmt args))))
+	    ((offsetof)
+	     (catch 'c99-error
+	       (lambda () (eval-offsetof tree udict))
 	       (lambda (key fmt . args) (apply fail fmt args))))
 	    ((ident) (or (eval-ident tree)
 			 (fail "cannot resolve identifier ~S" (sx-ref tree 1))))
