@@ -94,7 +94,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 pretty-print)
   #:re-export (*nyacc-version*)
-  #:version (1 05 0))
+  #:version (1 06 0))
 
 (define fh-cpp-defs
   (cond
@@ -611,7 +611,7 @@
 	(string-append "_" (number->string ix)))))
 
   (define (unitize decl seed)
-    (unitize-comp-decl decl seed #:namer anon-namer))
+    (dictize-comp-decl decl seed #:namer anon-namer))
 
   (let* ((fields (clean-fields (cdr field-list)))
 	 (uflds (reverse (fold unitize '() fields))))
@@ -786,18 +786,15 @@
   (let loop ((btail #f) (mxsz 0) (mxal 0) (flds fields))
     (if (null? flds) btail
 	(let ((mtail (cdr (udecl->mdecl (car flds)))))
-	  (call-with-values (lambda () (sizeof-mtail mtail #f)) ;; #f is arch
+	  (call-with-values (lambda () (sizeof-mtail mtail))
 	    (lambda (sz al)
 	      (if (> sz mxsz)
 		  (loop mtail sz (max al mxal) (cdr flds))
 		  (loop btail mxsz (max al mxal) (cdr flds)))))))))
 
-;;(else (fherr "can't deal with this alignment: ~S" align))))
-
 (define (clean-and-unitize-fields fields)
-  (map cdr (fold-right unitize-decl '() (clean-fields fields))))
+  (fold-right unitize-decl '() (clean-fields fields)))
 
-;;(define* (mtail->ffi-desc mdecl-tail #:optional in-compound)
 (export mtail->ffi-desc)
 (define* (mtail->ffi-desc mdecl-tail #:optional in-compound)
 
@@ -830,7 +827,7 @@
     
     (((struct-def (field-list . ,fields)))
      `(list ,@(map (lambda (fld)
-		     (let* ((udict (unitize-comp-decl fld))
+		     (let* ((udict (dictize-comp-decl fld))
 			    (name (caar udict))
 			    (udecl (cdar udict))
 			    (udecl (udecl-rem-type-qual udecl))
@@ -1171,9 +1168,11 @@
 			      (or (equal? node '(stor-spec (auto)))
 				  (equal? node '(stor-spec (register)))
 				  (equal? node '(stor-spec (static)))
+				  (equal? node '(type-qual (const)))
 				  (equal? node '(type-qual (volatile)))
 				  (equal? node '(type-qual (restrict)))))
 			    specl))
+
 	     (specl (if (fctn? declr)
 			(remove (lambda (node)
 				  (equal? node '(stor-spec (extern)))) specl)
@@ -1965,11 +1964,11 @@
 	 (cond
 	  ((and ;; Process the declaration if all conditions met:
 	    (declf name)		  ; 1) user wants it
-	    (not (member name defined)) ; 2) not already defined
-	    (not (and (pair? name)	  ; 3) not anonymous
-		      (string=? "*anon*" (cdr name)))))
+	    (not (member name defined))	  ; 2) not already defined
+	    (not (if (pair? name)	  ; 3) not anonymous 
+		     (string=? "*anon*" (cdr name))
+		     (string=? "" name))))
 	   (let ((udecl (udict-ref udict name)))
-	     ;;(sferr "processing ~S\n" name)
 	     (nlscm) (c99scm udecl)
 	     (if (*echo-decls*)
 		 (sfscm "(if echo-decls (display \"~A\\n\"))\n" name))
