@@ -18,8 +18,14 @@
 ;;; Notes:
 
 ;; 1) mdecl == munged (unwrapped) declaration
+;;    udecl : unitzied decl (not declr-list and one init-declr, etc
+;;    utent : (name . udecl) where name is string or pair (struct . "foo")
 ;; 2) Usual sequence is: expand-typerefs, stripdown-udecl, udecl->mdecl.
-;; 3) Unitize-decl is shallow.  It does not dive into structs and unitize.
+;; 3) unitize-decl is shallow.  It does not dive into structs and unitize.
+;; 4) expand-typerefs is shallow?  It does not dive into structs and unitize.
+;; 5) dictize replaces old unitize, w/ unitize making more sense now
+;;    a) unitize: (int (x y)) -> ((int x) (int y))
+;;    b) dictize: (int (x y)) -> (("x" . (int x)) ("y" . (int y)))
 
 ;; Todo:
 ;; 2) I want a way to keep named enums in expand-typerefs.
@@ -53,7 +59,8 @@
 	    typedef-decl?
 	    
 	    unitize-decl unitize-comp-decl unitize-param-decl
-	    declr-ident declr-id decl-id
+	    dictize-decl dictize-comp-decl dictize-param-decl
+	    decl-id
 	    iter-declrs
 	    split-decl
 
@@ -67,6 +74,7 @@
   #:re-export (expand-typerefs
 	       reify-declr reify-decl
 	       udecl->mdecl split-udecl
+	       declr-ident declr-name
 	       clean-field-list clean-fields)
   #:use-module (nyacc lang c99 cxeval)
   #:use-module (nyacc lang c99 munge-base)
@@ -187,7 +195,7 @@
        (lambda (tree seed)
 	 (cond
 	  ((eqv? (sx-tag tree) 'decl)
-	   (unitize-decl tree seed))
+	   (dictize-decl tree seed))
 	  ((inc-keeper? tree inc-filter) =>
 	   (lambda (inc-tree)
 	     (c99-trans-unit->udict inc-tree seed #:inc-filter inc-filter)))
@@ -209,9 +217,9 @@
   (if (pair? declrs)
       (fold-right
        (lambda (declr seed)
-	 (acons (declr-id declr) (sx-list tag attr specl declr) seed))
+	 (acons (declr-name declr) (sx-list tag attr specl declr) seed))
        seed declrs)
-      seed))
+      (acons "" (sx-list tag attr specl) seed)))
 
 ;; @deffn {Procedure} split-decl decl => values
 ;; This routine splits a declaration (or comp-decl or param-decl) into
@@ -239,7 +247,7 @@
 	 (declrs (and=> dclr-l sx-tail))) ; ((...))
     (values tag attr spec-l declrs)))
 
-;; @deffn {Procedure} unitize-decl decl [seed] [#:expand-enums #f] => seed
+;; @deffn {Procedure} dictize-decl decl [seed] [#:expand-enums #f] => seed
 ;; This is a fold iterator intended to be used by @code{c99-trans-unit->udict}.
 ;; It converts the multiple @code{init-declr} items in an @code{init-declr-list}
 ;; of a @code{decl} into an a-list of multiple pairs of name and @code{udecl}
@@ -271,27 +279,19 @@
 ;; struct and union (e.g., @code{__packed__}).  The latter is needed
 ;; because they appear in files under @file{/usr/include}.
 ;; @end deffn
-
-(define* (unitize-decl decl #:optional (seed '()))
+(define* (dictize-decl decl #:optional (seed '()))
   
   (define* (make-udecl type-tag attr guts #:optional typename)
     (if (and attr (pair? attr))
 	`(udecl (decl-spec-list (type-spec ,(cons* type-tag `(@ ,@attr) guts))))
 	`(udecl (decl-spec-list (type-spec ,(cons type-tag guts))))))
 
-  ;; update depends on whether unitize- procedures use fold or fold-right
-  (define (update-left name value tag attr specl declrs seed)
-    (acons name value (iter-declrs tag attr specl declrs seed)))
-
-  (define (update-right name value tag attr specl declrs seed)
+  (define (update name value tag attr specl declrs seed)
     (iter-declrs tag attr specl declrs (acons name value seed)))
 
-  (define update update-right)
-
-  ;;(simple-format #t "unitize-decl ~S\n" decl)
   (cond
    ((not (pair? decl))
-    (throw 'nyacc-error "unitize-decl: bad arg: ~S" decl))
+    (throw 'nyacc-error "dictize-decl: bad arg: ~S" decl))
    ((eqv? (sx-tag decl) 'udecl)
     (acons (udecl-id decl) decl seed))
 
@@ -384,11 +384,19 @@
 	(,_ (iter-declrs tag decl-attr specl declrs seed)))))
    
    ((eqv? (sx-tag decl) 'comp-udecl) (acons (udecl-id decl) decl seed))
-   ((eqv? (sx-tag decl) 'comp-decl) (unitize-comp-decl decl seed))
-   ((eqv? (sx-tag decl) 'param-decl) (unitize-param-decl decl seed))
+   ((eqv? (sx-tag decl) 'comp-decl) (dictize-comp-decl decl seed))
+   ((eqv? (sx-tag decl) 'param-decl) (dictize-param-decl decl seed))
    (else seed)))
 
-;; @deffn {Procedure} unitize-comp-decl decl [seed]
+(define* (unitize-decl decl #:optional (seed '()))
+  (fold-right
+   (lambda (ud-entry seed)
+     ;;(sferr "ue: ~S\n" (car ud-entry)) (pperr (cdr ud-entry))
+     ;;(pperr ud-entry)
+     (if (pair? (car ud-entry)) seed (cons (cdr ud-entry) seed)))
+   seed (dictize-decl decl)))
+
+;; @deffn {Procedure} dictize-comp-decl decl [seed] [#:namer namer]
 ;; This will turn
 ;; @example
 ;; (comp-decl (decl-spec-list (type-spec "int"))
@@ -406,19 +414,25 @@
 ;; functions @code{struct} and @code{union} field lists.  The result needs
 ;; to be reversed.
 ;; @end deffn
-(define* (unitize-comp-decl decl #:optional (seed '()))
+(define* (dictize-comp-decl decl #:optional (seed '()) #:key (namer def-namer))
   (cond
    ((not (pair? decl))
-    (throw 'nyacc-error "unitize-decl: bad arg: ~S" decl))
+    (throw 'nyacc-error "dictize-decl: bad arg: ~S" decl))
    ((eqv? (sx-tag decl) 'comp-udecl)
     (acons (udecl-id decl) decl seed))
    ((eqv? (sx-tag (sx-ref decl 2)) 'comp-declr-list)
     (let-values (((tag attr spec-l declrs) (split-decl decl)))
       (iter-declrs 'comp-udecl attr spec-l declrs seed)))
    (else
-    seed)))
+    (acons (namer) decl seed))))
 
-;; @deffn {Procedure} unitize-param-decl param-decl [seed] [#:expand-enums #f]
+(define* (unitize-comp-decl decl #:optional (seed '()) #:key (namer def-namer))
+  (fold-right
+   (lambda (ud-entry seed)
+     (if (pair? (car ud-entry)) seed (cons (cdr ud-entry) seed)))
+   seed (dictize-comp-decl decl #:namer namer)))
+
+;; @deffn {Procedure} dictize-param-decl param-decl [seed] [#:expand-enums #f]
 ;; This will turn
 ;; @example
 ;; (param-decl (decl-spec-list (type-spec "int"))
@@ -435,7 +449,7 @@
 ;; @*
 ;; TODO: What about abstract declarators?  Should use "*anon*".
 ;; @end deffn
-(define* (unitize-param-decl decl #:optional (seed '()) #:key (expand-enums #f))
+(define* (dictize-param-decl decl #:optional (seed '()))
   (if (not (eqv? 'param-decl (car decl))) seed
       (let* ((tag (sx-ref decl 0))
 	     (attr (sx-attr decl))
@@ -445,36 +459,18 @@
 	     (name (cadr ident)))
 	(acons name decl seed))))
 
-;; @deffn {Procedure} declr-ident declr => (ident "name")
-;; Given a declarator, aka @code{init-declr}, return the identifier.
-;; This is used by @code{trans-unit->udict}.
-;; @end deffn
-(define (declr-ident declr)
-  (sx-match declr
-    ((ident ,name) declr)
-    ((init-declr ,declr . ,rest) (declr-ident declr))
-    ((comp-declr ,declr) (declr-ident declr))
-    ((param-declr ,declr) (declr-ident declr))
-    ((ary-declr ,dir-declr ,array-spec) (declr-ident dir-declr))
-    ((ary-declr ,dir-declr) (declr-ident dir-declr))
-    ((ptr-declr ,pointer ,dir-declr) (declr-ident dir-declr))
-    ((ftn-declr ,dir-declr . ,rest) (declr-ident dir-declr))
-    ((scope ,declr) (declr-ident declr))
-    ((bit-field ,ident . ,rest) ident)
-    (,_ (throw 'c99-error "c99/munge: unknown declarator: ~S" declr))))
-
-;; @deffn {Procedure} declr-id decl => "name"
-;; This extracts the name from the return value of @code{declr-ident}.
-;; @end deffn
-(define (declr-id declr)
-  (and=> (declr-ident declr) cadr))
+(define* (unitize-param-decl decl #:optional (seed '()))
+  (fold-right
+   (lambda (ud-entry seed)
+     (if (pair? (car ud-entry)) seed (cons (cdr ud-entry) seed)))
+   seed (dictize-param-decl decl)))
 
 ;; @deffn {Procedure} udecl-id udecl => string
 ;; generate the name 
 ;; @end deffn
 (define (udecl-id udecl)
   ;; must be udecl w/ name
-  (declr-id (sx-ref udecl 2)))
+  (declr-name (sx-ref udecl 2)))
 
 ;; like member but returns first non-declr of type in dict
 (define (non-declr type udict)
@@ -776,11 +772,11 @@
   (let loop ((dsl1 '()) (const-seen? #f) (tail dsl-tail))
     (if (null? tail)
 	(reverse (if (and const-seen? keep-const?)
-		     (cons '(type-qual "const") dsl1)
+		     (cons '(type-qual (const)) dsl1)
 		     dsl1))
 	(case (caar tail)
 	  ((type-qual)
-	   (if (string=? (cadar tail) "const")
+	   (if (equal? (cadar tail) '(const))
 	       (loop dsl1 #t (cdr tail))
 	       (loop dsl1 const-seen? (cdr tail))))
 	  ((stor-spec)
